@@ -27,18 +27,50 @@ pub type MintSignatures = BTreeMap<DbcContentHash, (PublicKeySet, NodeSignature)
 
 pub const GENESIS_DBC_INPUT: Hash = Hash([0u8; 32]);
 
+pub trait SpendBook: std::fmt::Debug + Clone {
+    type Error: std::error::Error;
+
+    fn lookup(&self, dbc_hash: &DbcContentHash) -> Result<Option<&DbcTransaction>, Self::Error>;
+    fn log(
+        &mut self,
+        dbc_hash: DbcContentHash,
+        transaction: DbcTransaction,
+    ) -> Result<(), Self::Error>;
+
+    fn entries(&self) -> Box<dyn Iterator<Item = (&DbcContentHash, &DbcTransaction)> + '_>;
+}
+
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
-pub struct SpendBook {
+pub struct SimpleSpendBook {
     pub transactions: BTreeMap<DbcContentHash, DbcTransaction>,
 }
 
-impl SpendBook {
-    fn lookup(&self, dbc_hash: &DbcContentHash) -> Option<&DbcTransaction> {
-        self.transactions.get(dbc_hash)
+impl SpendBook for SimpleSpendBook {
+    type Error = std::convert::Infallible;
+
+    fn lookup(&self, dbc_hash: &DbcContentHash) -> Result<Option<&DbcTransaction>, Self::Error> {
+        Ok(self.transactions.get(dbc_hash))
     }
 
-    fn log(&mut self, dbc_hash: DbcContentHash, transaction: DbcTransaction) {
+    fn log(
+        &mut self,
+        dbc_hash: DbcContentHash,
+        transaction: DbcTransaction,
+    ) -> Result<(), Self::Error> {
         self.transactions.insert(dbc_hash, transaction);
+        Ok(())
+    }
+
+    fn entries(&self) -> Box<dyn Iterator<Item = (&DbcContentHash, &DbcTransaction)> + '_> {
+        Box::new(self.transactions.iter())
+    }
+}
+
+impl SimpleSpendBook {
+    pub fn new() -> Self {
+        Self {
+            transactions: Default::default(),
+        }
     }
 }
 
@@ -122,20 +154,21 @@ pub struct ReissueRequest {
         HashMap<DbcContentHash, (threshold_crypto::PublicKey, threshold_crypto::Signature)>,
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct Mint<K>
+#[derive(Debug, Clone, Deserialize, Serialize)]
+pub struct Mint<K, S>
 where
     K: KeyManager,
+    S: SpendBook,
 {
     pub(crate) key_manager: K,
-    pub spendbook: SpendBook,
+    pub spendbook: S,
 }
 
-impl<K: KeyManager> Mint<K> {
-    pub fn new(key_manager: K) -> Self {
+impl<K: KeyManager, S: SpendBook> Mint<K, S> {
+    pub fn new(key_manager: K, spendbook: S) -> Self {
         Self {
             key_manager,
-            spendbook: Default::default(),
+            spendbook,
         }
     }
 
@@ -158,12 +191,18 @@ impl<K: KeyManager> Mint<K> {
             outputs: BTreeSet::from_iter(vec![content.hash()]),
         };
 
-        match self.spendbook.lookup(&GENESIS_DBC_INPUT) {
+        match self
+            .spendbook
+            .lookup(&GENESIS_DBC_INPUT)
+            .map_err(|e| Error::SpendBook(e.to_string()))?
+        {
             Some(tx) if tx != &transaction => return Err(Error::GenesisInputAlreadySpent),
             _ => (),
         }
 
-        self.spendbook.log(GENESIS_DBC_INPUT, transaction.clone());
+        self.spendbook
+            .log(GENESIS_DBC_INPUT, transaction.clone())
+            .map_err(|e| Error::SpendBook(e.to_string()))?;
         let transaction_sig = self
             .key_manager
             .sign(&transaction.hash())
@@ -181,8 +220,12 @@ impl<K: KeyManager> Mint<K> {
         ))
     }
 
-    pub fn is_spent(&self, dbc_hash: DbcContentHash) -> bool {
-        self.spendbook.lookup(&dbc_hash).is_some()
+    pub fn is_spent(&self, dbc_hash: DbcContentHash) -> Result<bool> {
+        Ok(self
+            .spendbook
+            .lookup(&dbc_hash)
+            .map_err(|e| Error::SpendBook(e.to_string()))?
+            .is_some())
     }
 
     pub fn key_manager(&self) -> &K {
@@ -214,7 +257,12 @@ impl<K: KeyManager> Mint<K> {
 
         // Validate that each input has not yet been spent.
         for input in inputs_belonging_to_mint.iter() {
-            if let Some(transaction) = self.spendbook.lookup(&input).cloned() {
+            if let Some(transaction) = self
+                .spendbook
+                .lookup(&input)
+                .map_err(|e| Error::SpendBook(e.to_string()))?
+                .cloned()
+            {
                 // This input has already been spent, return the spend transaction to the user
                 let transaction_sigs = self.sign_transaction(&transaction)?;
                 return Err(Error::DbcAlreadySpent {
@@ -232,7 +280,9 @@ impl<K: KeyManager> Mint<K> {
             .iter()
             .filter(|&i| inputs_belonging_to_mint.contains(&i.name()))
         {
-            self.spendbook.log(input.name(), transaction.clone());
+            self.spendbook
+                .log(input.name(), transaction.clone())
+                .map_err(|e| Error::SpendBook(e.to_string()))?;
         }
 
         Ok((transaction, transaction_sigs))
@@ -261,12 +311,12 @@ impl<K: KeyManager> Mint<K> {
     }
 
     // Used in testing / benchmarking
-    pub fn snapshot_spendbook(&self) -> SpendBook {
+    pub fn snapshot_spendbook(&self) -> S {
         self.spendbook.clone()
     }
 
     // Used in testing / benchmarking
-    pub fn reset_spendbook(&mut self, spendbook: SpendBook) {
+    pub fn reset_spendbook(&mut self, spendbook: S) {
         self.spendbook = spendbook
     }
 }
@@ -293,7 +343,7 @@ mod tests {
             ),
             genesis_owner.public_key_set.public_key(),
         );
-        let mut genesis_node = Mint::new(key_manager);
+        let mut genesis_node = Mint::new(key_manager, SimpleSpendBook::new());
 
         let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
             genesis_node.issue_genesis_dbc(1000).unwrap();
@@ -330,7 +380,7 @@ mod tests {
             ),
             genesis_owner.public_key_set.public_key(),
         );
-        let mut genesis_node = Mint::new(key_manager.clone());
+        let mut genesis_node = Mint::new(key_manager.clone(), SimpleSpendBook::new());
 
         let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
             genesis_node.issue_genesis_dbc(output_amount).unwrap();
@@ -437,7 +487,7 @@ mod tests {
             ),
             genesis_owner.public_key_set.public_key(),
         );
-        let mut genesis_node = Mint::new(key_manager);
+        let mut genesis_node = Mint::new(key_manager, SimpleSpendBook::new());
 
         let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
             genesis_node.issue_genesis_dbc(1000).unwrap();
@@ -571,7 +621,7 @@ mod tests {
             ),
             genesis_owner.public_key_set.public_key(),
         );
-        let mut genesis_node = Mint::new(key_manager);
+        let mut genesis_node = Mint::new(key_manager, SimpleSpendBook::new());
 
         let genesis_amount: u64 = input_amounts.iter().sum();
         let (gen_dbc_content, gen_dbc_trans, (_gen_key, gen_node_sig)) =
@@ -855,7 +905,7 @@ mod tests {
             ),
             genesis_owner.public_key_set.public_key(),
         );
-        let mut genesis_node = Mint::new(key_manager);
+        let mut genesis_node = Mint::new(key_manager, SimpleSpendBook::new());
 
         let input_owner = crate::bls_dkg_id();
         let input_content = DbcContent::new(
